@@ -1,4 +1,4 @@
-from typing import Any, Dict, Optional
+from typing import Any, Callable, Dict, List, Optional
 
 from xpander_sdk.models.frameworks import Framework
 from xpander_sdk.modules.agents.sub_modules.agent import Agent
@@ -7,7 +7,8 @@ from xpander_sdk.modules.tasks.sub_modules.task import Task
 async def dispatch_get_args(
     agent: Agent,
     task: Optional[Task] = None,
-    override: Optional[Dict[str, Any]] = None
+    override: Optional[Dict[str, Any]] = None,
+    tools: Optional[List[Callable]] = None,
 ) -> Dict[str, Any]:
     """
     Dispatch to the correct framework-specific argument resolver.
@@ -16,6 +17,7 @@ async def dispatch_get_args(
         agent (Agent): The agent to generate args for.
         task (Optional[Task]): Optional runtime task.
         override (Optional[Dict[str, Any]]): Dict of override values.
+        tools (Optional[List[Callable]]): Optional additional tools to be added to the agent arguments.
 
     Returns:
         Dict[str, Any]: Arguments for instantiating the framework agent.
@@ -23,7 +25,7 @@ async def dispatch_get_args(
     match agent.framework:
         case Framework.Agno:
             from .agno import build_agent_args
-            return await build_agent_args(xpander_agent=agent, task=task, override=override)
+            return await build_agent_args(xpander_agent=agent, task=task, override=override, tools=tools)
         # case Framework.Langchain: # PLACEHOLDER
         #     from .langchain import build_agent_args
         #     return await build_agent_args(xpander_agent=agent, task=task, override=override)
